d3.csv('assets/data/abschluesse.csv', display); // Daten laden
setupButtons(); // Button Setup

//* ------------------------------------------------------------------
//
// Teil 1 - Allgemeiner Code
//
// Initialisierungs-Code und Helfer-Funktionen um eine neue Bubble-Chart-Instanz zu 
// erstellen, um die Daten zu laden und um die Daten darzustellen
//
// -----------------------------------------------------------------*/

var myBubbleChart = bubbleChart();

/* Funktion display ruft die Bubble-Chart Funktion auf und stellt sie im #vis div dar. Wird nach dem laden der Daten aus dem CSV gecallt. */
function display(error, data) {
  if (error) {
    console.log(error);
  }
    var clusteredData = clusterData(data);
  myBubbleChart('#vis', clusteredData);
}

/*
Clusters data based on common attributes and adds additional amount attibute.
*/
function clusterData(data) {
    var parsedData = [];

    data.forEach(function(el){
        var matching = false;
        //Check if row already included in parsedData. If so, increase amount else add new row
        for(var i = 0; i < parsedData.length; i ++){
            var parsedEl = parsedData[i];
            //Compare attributes of parsedElement and csv data el and increase amount if matching
            /*
            Following attributes are not taken into account:
								- id
            */
            if(el.dauer == parsedEl.dauer && el.jahr == parsedEl.jahr && el.bezirkbezeichnung == parsedEl.bezirkbezeichnung && el.bezirk == parsedEl.bezirk && el.wochentag == parsedEl.wochentag && el.linie == parsedEl.linie && el.geschlecht == parsedEl.geschlecht && el.vorfall == parsedEl.vorfall && el.vorfall_text == parsedEl.vorfall_text && el.meldung == parsedEl.meldung && el.linienbezeichnung == parsedEl.linienbezeichnung){
                parsedEl.anzahl ++;
                matching = true;
                break;
           }
        }
        //Insert if not matching any element of the parsedData array
        if(!matching){
            el.anzahl = 1;
            parsedData.push(el);
        }
    });
    return parsedData;
             
}

/* Setup der Layout Buttons damit zwischen den Ansichten getogglet werden kann */
function setupButtons() {
  d3.select('#toolbar')
    .selectAll('.button')
    .on('click', function () {
      d3.selectAll('.button').classed('active', false); // Remove active class from all buttons 
      var button = d3.select(this); // Find the button just clicked
      button.classed('active', true); // Set it as the active button
      var buttonId = button.attr('id'); // Get the id of the button
      myBubbleChart.toggleDisplay(buttonId); // Toggle the bubble chart based on the currently clicked button.
    });
}

/* Helper-Funktion zum konvertieren von Zahlen in einen String mit Kommas für schönere Darstellung */
function addCommas(nStr) {
  nStr += '';
  var x = nStr.split('.');
  var x1 = x[0];
  var x2 = x.length > 1 ? '.' + x[1] : '';
  var rgx = /(\d+)(\d{3})/;
  while (rgx.test(x1)) {
    x1 = x1.replace(rgx, '$1' + ',' + '$2');
  }
  return x1 + x2;
}

//* ------------------------------------------------------------------
//
// Teil 2 - Funktion für die Erstellung der Bubble Chart
//
// -----------------------------------------------------------------*/

/* Funktion für die Erstellung der Bubble Chart. Returned eine Funktion, welche eine neue Bubble Chart erstellt, gegeben ein DOM-Element zur Darstellung und gegeben ein Datenset zur Visualisierung. */

function bubbleChart() {
  var width = 1300; // Konstanten für die Grösse
  var height = 900; // Konstanten für die Grösse
  var center = { x: width / 2, y: height / 2 };  // Locations to move bubbles towards, depending on which view mode is selected.

//* ------------------------------------------------------------------
//
// Teil 3 - Beschriftungen
//
// -----------------------------------------------------------------*/
	
// Abschlüsse nach Schwerpunktfach
    
  var lineCenters = { // Center locations of the bubbles. 
    10311000: { x: 350, y: 450 },
    10312000: { x: 520, y: 490 },
    10313000: { x: 900, y: 270 },
    10314000: { x: 670, y: 290 },
   	10315000: { x: 830, y: 450 },
		10317000: { x: 450, y: 290 },
		10318000: { x: 280, y: 260 },
  };

  var linesTitleX = {  // X locations of the course titles.
    'Alte Sprachen': 210,
    'Eine moderne Sprache': 510,
    'Physik und Anwendungen der Mathematik': 990,
    'Biologie und Chemie': 690,
    'Wirtschaft und Recht': 920,
    'Bildnerisches Gestalten': 380,
		'Musik': 150,
  };
	
	 var linesTitleY = {  // Y locations of the course titles.
    'Alte Sprachen': 340,
    'Eine moderne Sprache': 340,
    'Physik und Anwendungen der Mathematik': 65,
    'Biologie und Chemie': 65,
    'Wirtschaft und Recht': 340,
    'Bildnerisches Gestalten': 65,
		'Musik': 65,
  };
	
 
// Abschlüsse nach Geschlecht
    
var yearsCenters = { // Center locations of the bubbles.
    'weiblich': { x: 500, y: 340 },
		'maennlich': { x: 770, y: 340 },
  };
	
  var yearsTitleX = { // X locations of the gender titles.
		'weiblich': 430,
		'männlich': 850
  };

  var yearsTitleY = { // Y locations of the gender titles.
		'weiblich': 65,
		'männlich': 65
  };
    
// Abschlüsse nach Wohnbezirke

var durationCenters = { // Center locations of the bubbles.
    101: { x: 240, y: 340 },
    102: { x: 290, y: 340 },
    103: { x: 350, y: 340 },
    104: { x: 410, y: 340 },
		105: { x: 480, y: 340 },
		106: { x: 550, y: 340 },
    107: { x: 620, y: 340 },
    108: { x: 690, y: 340 },
    109: { x: 760, y: 340 },
		110: { x: 830, y: 340 },
		111: { x: 900, y: 340 },
		112: { x: 970, y: 340 },
		'NULL': { x: 550, y: 500 }
};

  var durationTitleX = { // X locations of the district titles.
   	'Affoltern': 50,
    'Andelfingen': 140,
    'Bülach': 230,
    'Dielsdorf': 320,
		'Hinwil': 400,
		'Horgen': 490,
    'Meilen': 600,
    'Pfäffikon': 700,
    'Uster': 780,
		'Winterthur': 900,
		'Dietikon': 990,
		'Zürich': 1100,
		'Ausserkantonal': 530	
  };
	
	  var durationTitleY = { // Y locations of the district titles.
   	'Affoltern': 150,
    'Andelfingen': 130,
    'Bülach': 150,
    'Dielsdorf': 130,
		'Hinwil': 150,
		'Horgen': 130,
    'Meilen': 150,
    'Pfäffikon': 130,
    'Uster': 150,
		'Winterthur': 130,
		'Dietikon': 150,
		'Zürich': 130,
		'Ausserkantonal': 530	
  };
    
// Abschlüsse nach Kantonsschulen
    
  var typeCenters = { // Center locations of the bubbles. 
    29850: { x: 300, y: 210 },	
    29875: { x: 450, y: 230 },
    29855: { x: 580, y: 250 },
    29845: { x: 710, y: 235 },
    29820: { x: 840, y: 220 },
    29830: { x: 320, y: 360 },
    29870: { x: 460, y: 360 },
    29880: { x: 580, y: 350 },
    29900: { x: 700, y: 360 },
    29805: { x: 830, y: 360 },
    29815: { x: 320, y: 495 },
    29810: { x: 460, y: 480 },
    29865: { x: 600, y: 480 },
		29825: { x: 730, y: 490 },
    29890: { x: 820, y: 490 },
    29840: { x: 300, y: 640 },
		29885: { x: 460, y: 640 },
    29895: { x: 650, y: 630 },
    29860: { x: 840, y: 650 },
  };

  var typeTitleX = {  // X locations of the schools.
    'Liceo Artistico Zürich': 200,
    'KS Büelrain Winterthur': 400,
    'KS Enge Zürich':  570,
    'KS Freudenberg Zürich': 740,
    'KS Hohe Promenade Zürich': 930,
    'KS Hottingen Zürich': 200,
    'KS Im Lee Winterthur': 400,
    'KS Küsnacht': 570,
    'KS Limmattal Urdorf': 740,
    'KS Rämibühl LG Zürich': 930,
    'KS Rämibühl MNG Zürich': 200,
    'KS Rämibühl RG Zürich': 400,
    'KS Rychenberg Winterthur': 590,
		'KS Stadelhofen Zürich': 790,
    'KS Uster': 940,
    'KS Wiedikon Zürich': 200,
    'KS Zürcher Oberland Wetzikon': 420,
    'KS Zürcher Unterland Bülach': 680,
    'KS Zürich Nord': 930
    
  };
 
  var typeTitleY = {  // Y locations of the schools.
    'Liceo Artistico Zürich': 45,
    'KS Büelrain Winterthur': 45,
    'KS Enge Zürich':  45,
    'KS Freudenberg Zürich': 45,
    'KS Hohe Promenade Zürich': 45,
    'KS Hottingen Zürich': 220,
    'KS Im Lee Winterthur': 220,
    'KS Küsnacht': 220,
    'KS Limmattal Urdorf': 220,
    'KS Rämibühl LG Zürich': 220,
    'KS Rämibühl MNG Zürich': 420,
    'KS Rämibühl RG Zürich': 420,
    'KS Rychenberg Winterthur': 420,
		'KS Stadelhofen Zürich': 420,
    'KS Uster': 420,
    'KS Wiedikon Zürich': 630,
    'KS Zürcher Oberland Wetzikon': 630,
    'KS Zürcher Unterland Bülach': 630,
    'KS Zürich Nord': 630,
  };
	
// Abschlüsse nach Nationalität
    
  var weekdayCenters = { // Center locations of the bubbles.  
    1: { x: 360, y: 280 },
    2: { x: 580, y: 280 },
    3: { x: 350, y: 430 },
    4: { x: 790, y: 320 },
    5: { x: 770, y: 430 },
    6: { x: 960, y: 230 },
		7: { x: 760, y: 220 },
    8100: { x: 580, y: 480 }
  };

  var weekdayTitleX = {  // X locations of the nationalities.
    'Nord- und Westeuropa sowie Nordamerika': 250,
    'Ost- und Süd-Osteuropa': 600,
    'Süd- und Süd_Westeuropa': 200,
    'Afrika': 920,
    'Lateinamerika und Karibik': 920,
    'Asien': 1080,
    'Ozeanien': 850,
		'Schweiz': 590,
  };
	
	var weekdayTitleY = {  // Y locations of nationalites.
    'Nord- und Westeuropa sowie Nordamerika': 65,
    'Ost- und Süd-Osteuropa': 65,
    'Süd- und Süd_Westeuropa': 350,
    'Afrika': 200,
    'Lateinamerika und Karibik': 340,
    'Asien': 100,
    'Ozeanien': 65,
		'Schweiz': 220,
  };
    
//* ------------------------------------------------------------------
//
// Teil 4 - Datenmanipulation (csv into JS)
//
// -----------------------------------------------------------------*/
    
// Used when setting up force and moving around nodes
  var damper = 0.102;

// These will be set in create_nodes and create_vis
  var svg = null;
  var bubbles = null;
  var nodes = [];

/* Charge function that is called for each node. Charge is proportional to the diameter of the circle (which is stored in the radius attribute of the circle's associated data. This is done to allow for accurate collision detection with nodes of different sizes. Charge is negative because we want nodes to repel. Dividing by 8 scales down the charge to be appropriate for the visualization dimensions. */
    
  function charge(d) {
    return -Math.pow(d.radius, 2.0) / 4;
  }

/* Here we create a force layout and configure it to use the charge function from above. This also sets some contants to specify how the force layout should behave. More configuration is done below. */
    
  var force = d3.layout.force()
    .size([width, height])
    .charge(charge)
    .gravity(-0.01)
    .friction(0.9);
    
    force.on('start', forceStartCallback);


  // Sizes bubbles based on their area instead of raw radius
  var radiusScale = d3.scale.pow()
    .exponent(0.5)
    .range([2, 20]);

/* This data manipulation function takes the raw data from the CSV file and converts it into an array of node objects. Each node will store data and visualization values to visualize a bubble. rawData is expected to be an array of data objects, read in from one of d3's loading functions like d3.csv. This function returns the new node array, with a node in that array for each element in the rawData input. */
    
  function createNodes(rawData) {
 
/* Use map() to convert raw data into node data. Checkout http://learnjsdata.com/ for more on working with data. */
      
    var myNodes = rawData.map(function (d) {
      return {
        id: d.id,
        radius: radiusScale(+d.anzahl), // Berechnung Radius für bubbles
        value: d.anzahl, // Ansicht nach Dauer
        group: d.linie, // Darstellung
        duration: d.bezirk,
        line: d.linie,
        years: d.geschlecht,
        type: d.vorfall,
        datum: d.jahr,
        weekday: d.wochentag,
        meldung: d.meldung, 
        linienbez: d.linienbezeichnung,  
        vorfalltext: d.vorfall_text,
				bezirkbezeichnung: d.bezirkbezeichnung,
        anzahl: d.anzahl,
        x: Math.random() * 800,
        y: Math.random() * 900
      };
    });

    // sort them to prevent occlusion of smaller nodes.
    myNodes.sort(function (a, b) { return b.value - a.value; });

    return myNodes;
  }

/* Main entry point to the bubble chart. This function is returned by the parent closure. It prepares the rawData for visualization and adds an svg element to the provided selector and starts the visualization creation process. selector is expected to be a DOM element or CSS selector that points to the parent element of the bubble chart. Inside this element, the code will add the SVG container for the visualization. rawData is expected to be an array of data objects as provided by a d3 loading function like d3.csv. */
    
  var chart = function chart(selector, rawData) {
    
/* Use the max duration in the data as the max in the scale's domain. note we have to ensure the duration is a number by converting it with `+`. */

    var maxAmount = d3.max(rawData, function (d) { return +d.anzahl; });
    radiusScale.domain([0, maxAmount*1]);

    nodes = createNodes(rawData);
    // Set the force's nodes to our newly created nodes array.
    force.nodes(nodes);

/* Create a SVG element inside the provided selector with desired size. */
      
    svg = d3.select(selector)
      .append('svg')
      .attr('width', width)
      .attr('height', height);

    // Bind nodes data to what will become DOM elements to represent them.
    bubbles = svg.selectAll('.bubble')
      .data(nodes, function (d) { return d.id; });

    // Create new circle elements each with class `bubble`.
    // There will be one circle.bubble for each object in the nodes array.
    // Initially, their radius (r attribute) will be 0.
    bubbles.enter().append('circle')
      .classed('bubble', true)
      .attr('r', 0)
      .attr('fill', function (d) { return fillColor(d.line); })
      .attr('stroke', function (d) { return d3.rgb(fillColor(d.line)).darker(); })
      .attr('stroke-width', 2)
      .on('mouseover', showDetail)
      .on('mouseout', hideDetail);

    // Fancy transition to make bubbles appear, ending with the
    // correct radius
    bubbles.transition()
      .duration(5000)
      .attr('r', function (d) { return d.radius; });

    // initiales layout = single group.
    groupBubbles();
  };
    
    function forceStartCallback(){
    var ticksPerRender = 1;
    requestAnimationFrame(function render() {

  for (var i = 0; i < ticksPerRender; i++) {
    force.tick();
  }

  // UPDATE NODE AND LINK POSITIONS HERE

  if (force.alpha() > 0) {
    requestAnimationFrame(render);
  }
});
    }

//* ------------------------------------------------------------------
//
// Teil 5 - Initiale Ansicht, alle Abschlüsse: "Single group mode"
//-> exakt so aus dem .json files von Bernmobil übernommen, die Variablennamen bleiben wie bei Bernmobil bestehen.  
// -----------------------------------------------------------------*/

/* Sets visualization in "single group mode". The other labels are hidden and the force layout tick function is set to move all nodes to the center of the visualization. */
    
  function groupBubbles() {
    hideLines();
    hideYears();
    hideDuration();
    hideType();
    hideWeekday();

    force.on('tick', function (e) {
      bubbles.each(moveToCenter(e.alpha))
        .attr('cx', function (d) { return d.x; })
        .attr('cy', function (d) { return d.y; });
    });

    force.start();
  }

/* Helfer-Funktion für den "single group mode". Returned eine Funktion, welche die Daten nimmt für einen Node und die Positionsdaten des Nodes so anpasst, dass er in die Mitte der Visualisierung geht.  

Die Positionierung basiert auf dem alpha Parameter des force layouts und wird kleiner, je länger das force layout läuft. Damit wird die bewegung der nodes verringert, je näher sie ihrem Ziel sind und erlaubt so anderen kräften wie der anziehungskraft der nodes auch die finale Positionen zu bestimmen. */
    
  function moveToCenter(alpha) {
    return function (d) {
      d.x = d.x + (center.x - d.x) * damper * alpha;
      d.y = d.y + (center.y - d.y) * damper * alpha;
    };
  }

//* ------------------------------------------------------------------
//
// Teil 6 - Abschlüsse nach Scherpunktfach
//
// -----------------------------------------------------------------*/
    
  function splitBubblesintoLines() {
    showLines();
    hideYears();
    hideDuration();
    hideType();
    hideWeekday();

    force.on('tick', function (e) {
      bubbles.each(moveToLines(e.alpha))
        .attr('cx', function (d) { return d.x; })
        .attr('cy', function (d) { return d.y; });
    });

    force.start();
  } 
    
  function moveToLines(alpha) {
    return function (d) {
      var target = lineCenters[d.line];
      d.x = d.x + (target.x - d.x) * damper * alpha * 1.1;
      d.y = d.y + (target.y - d.y) * damper * alpha * 1.1;
    };
  }
    
  function hideLines() {
    svg.selectAll('.line').remove();
  }

  function showLines() {
      
    var linesData = d3.keys(linesTitleX);
    var lines = svg.selectAll('.line')
      .data(linesData);

    lines.enter().append('text')
      .attr('class', 'line')
      .attr('x', function (d) { return linesTitleX[d]; })
				.attr('y', function (d) { return linesTitleY[d]; })
     
      .attr('text-anchor', 'middle')
      .text(function (d) { return d; });
  }

//* ------------------------------------------------------------------
//
// Teil 7 - Abschlüsse nach Geschlecht
//
// -----------------------------------------------------------------*/
 
  function splitBubblesintoYears() {
    showYears();
    hideLines();
    hideDuration();
    hideType();
    hideWeekday();

    force.on('tick', function (e) {
      bubbles.each(moveToYears(e.alpha))
        .attr('cx', function (d) { return d.x; })
        .attr('cy', function (d) { return d.y; });
    });

    force.start();
  }
    
  function moveToYears(alpha) {
    return function (d) {
      var target = yearsCenters[d.years];
      d.x = d.x + (target.x - d.x) * damper * alpha * 1.1;
      d.y = d.y + (target.y - d.y) * damper * alpha * 1.1;
    };
  }

  function hideYears() {
    svg.selectAll('.years').remove();
  }

  function showYears() {

    var yearsData = d3.keys(yearsTitleX);
    var years = svg.selectAll('.years')
      .data(yearsData);

    years.enter().append('text')
      .attr('class', 'years')
      .attr('x', function (d) { return yearsTitleX[d]; })
       .attr('y', function (d) { return yearsTitleY[d]; })
      .attr('text-anchor', 'middle')
      .text(function (d) { return d; });
  }

//* ------------------------------------------------------------------
//
// Teil 8 - Abschlüsse nach Bezirke
//
// -----------------------------------------------------------------*/
 
  function splitBubblesintoDuration() {
    showDuration();
    hideYears();
    hideLines();
    hideType();
    hideWeekday();

    force.on('tick', function (e) {
      bubbles.each(moveToDuration(e.alpha))
        .attr('cx', function (d) { return d.x; })
        .attr('cy', function (d) { return d.y; });
    });

    force.start();
  }
    
  function moveToDuration(alpha) {
    return function (d) {
      var target = durationCenters[d.duration];
      d.x = d.x + (target.x - d.x) * damper * alpha * 1.1;
      d.y = d.y + (target.y - d.y) * damper * alpha * 1.1;
    };
  }

  function hideDuration() {
    svg.selectAll('.duration').remove();
  }

  function showDuration() {

    var durationData = d3.keys(durationTitleX);
    var duration = svg.selectAll('.duration')
      .data(durationData);

    duration.enter().append('text')
      .attr('class', 'duration')
      .attr('x', function (d) { return durationTitleX[d]; })
     .attr('y', function (d) { return durationTitleY[d]; })
      .attr('text-anchor', 'middle')
      .text(function (d) { return d; });
    }
  
//* ------------------------------------------------------------------
//
// Teil 9 - Abschlüsse nach Kantonsschule
//
// -----------------------------------------------------------------*/
    
  function splitBubblesintoType() {
    showType();
    hideYears();
    hideLines();
    hideDuration();
    hideWeekday();
      force.on('tick', function (e) {
      bubbles.each(moveToType(e.alpha))
        .attr('cx', function (d) { return d.x; })
        .attr('cy', function (d) { return d.y; });
    });
    force.start();
  }
    
  function moveToType(alpha) {
    return function (d) {
      var target = typeCenters[d.type];
      d.x = d.x + (target.x - d.x) * damper * alpha * 1.1;
      d.y = d.y + (target.y - d.y) * damper * alpha * 1.1;
    };
  }

  function hideType() {
    svg.selectAll('.type').remove();
  }

  function showType() {
 
    var typeData = d3.keys(typeTitleX);
    var type = svg.selectAll('.type')
      .data(typeData);

    type.enter().append('text')
      .attr('class', 'type')
      .attr('x', function (d) { return typeTitleX[d]; })
      .attr('y', function (d) { return typeTitleY[d]; })
      .attr('text-anchor', 'middle')
      .text(function (d) { return d; });
    }

//* ------------------------------------------------------------------
//
// Teil 10 - Abschlüsse nach Nationalität
//
// -----------------------------------------------------------------*/
    
  function splitBubblesintoWeekday() {
    showWeekday();
    hideType();
    hideYears();
    hideLines();
    hideDuration();

    force.on('tick', function (e) {
      bubbles.each(moveToWeekday(e.alpha))
        .attr('cx', function (d) { return d.x; })
        .attr('cy', function (d) { return d.y; });
    });

    force.start();
  }

  function moveToWeekday(alpha) {
    return function (d) {
      var target = weekdayCenters[d.weekday];
      d.x = d.x + (target.x - d.x) * damper * alpha * 1.1;
      d.y = d.y + (target.y - d.y) * damper * alpha * 1.1;
    };
  }

  function hideWeekday() {
    svg.selectAll('.weekday').remove();
  }

  function showWeekday() {

    var weekdayData = d3.keys(weekdayTitleX);
    var type = svg.selectAll('.weekday')
      .data(weekdayData);

    type.enter().append('text')
      .attr('class', 'weekday')
      .attr('x', function (d) { return weekdayTitleX[d]; })
      	.attr('y', function (d) { return weekdayTitleY[d]; })
      .attr('text-anchor', 'middle')
      .text(function (d) { return d; });
    }

//* ------------------------------------------------------------------
//
// Teil 11 - Wechseln zwischen den Ansichten
//
// -----------------------------------------------------------------*/
    
  /* Externally accessible function (this is attached to the returned chart function). Allows the visualization to toggle between "single group" and "split by ..." modes. */

  chart.toggleDisplay = function (displayName) {
    if (displayName === 'line') {
      splitBubblesintoLines();
    } else if (displayName === 'years') {
      splitBubblesintoYears();
    } else if (displayName === 'duration') {
      splitBubblesintoDuration();
    } else if (displayName === 'type') {
      splitBubblesintoType();
    } else if (displayName === 'weekday') {
      splitBubblesintoWeekday();
    } else {
      groupBubbles();
    }
  };

  chart.toggleDisplay2 = function (displayName2) {
    if (displayName === 'line') {
      splitBubblesintoLines();
    } else {
      groupBubbles();
    }
  };
    
  return chart;
}

// Ende der Funktion

//* ------------------------------------------------------------------
//
// Teil 12 - Tooltip
//
// -----------------------------------------------------------------*/

  // Tooltip für Mousover
  var tooltip2 = floatingtooltip2('gates_tooltip2', 240);

  var fillColor = d3.scale.ordinal()
    .domain(['10311000', '10312000', '10313000', '10314000', '10315000', '10317000', '10318000'])
    .range(['#CEF6D8', '#04B431',  '#CD003C', '#FF5722', '#FFEB3B',  '#303F9F', '#03A9F4']);

  /* Tooltip-Funktion*/
  function showDetail(d) {

    d3.select(this).attr('stroke', 'black');

    var content = '<span class="name">Kantonsschule: </span><span class="value">' +
                  d.vorfalltext +
									'</span><br/>' +
                  '<span class="name">Abschluss: </span><span class="value">' +
                  d.linienbez +
                   '</span><br/>' +
                  '<span class="name">Wohnbezirk: </span><span class="value">' +
                  d.bezirkbezeichnung +
									  '</span><br/>' +
                  '<span class="name">Abschlussjahr: </span><span class="value">' +
                  d.datum +
									'</span><br/>' +
                  '<span class="name">Geschlecht: </span><span class="value">' +
                  d.years +
                  '</span><br/>' +
                  '<span class="name"> Nationalität: </span><span class="value">' +
                  d.meldung +
                  '</span><br/>' +
		'<span class="name"> Anzahl: </span><span class="value">' +
                  d.anzahl +
                  '</span>';
    tooltip2.showtooltip2(content, d3.event);
  }

  function hideDetail(d) { // tooltip verstecken

    d3.select(this)
      .attr('stroke', d3.rgb(fillColor(d.line)).darker());

    tooltip2.hidetooltip2();
  }

//* ------------------------------------------------------------------
//
// The End
//
// -----------------------------------------------------------------*/